package cn.com.ctop.okr.controller;

import cn.com.ctop.okr.dto.OkrDto;
import cn.com.ctop.okr.dto.UserInfoDto;
import cn.com.ctop.okr.entity.*;
import cn.com.ctop.okr.service.OkrService;
import cn.com.ctop.okr.vo.Result;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/okr")
public class OkrController {
    private static Logger logger = Logger.getLogger(OkrController.class);
    @Autowired
    private OkrService okrService;

    @ResponseBody
    @RequestMapping(value = "/insertTaskInfo")
    public Result insertTaskInfo(@RequestBody OkrDto dto ,
                                 HttpServletRequest request,
                                 HttpServletResponse response){

        logger.info("okr/insertTaskInfo 方法开始");
        Result result = new Result();
        result.setSuccess(true);

        OkrTaskInfo taskInfo = dto.getTaskInfo();
        if(dto == null || dto.getTaskInfo() == null || StringUtils.isBlank(taskInfo.getTaskTitle()) ||
          taskInfo.getType() == null || taskInfo.getPriority() == null
        ){
            result.setSuccess(false);
            result.setMessage("任务标题、类型、优先级均不可为空");
            return result;
        }

        try {
            okrService.insertTaskInfo(taskInfo);
        }catch(Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally {
            logger.info("okr/insertTaskInfo 方法结束");
        }

        return result;

    }

    @ResponseBody
    @RequestMapping(value = "/updateTaskInfo")
    public Result updateTaskInfo(@RequestBody OkrDto dto,
                                 HttpServletRequest request,
                                 HttpServletResponse response){

        logger.info("okr/updateTaskInfo 方法开始");
        Result result = new Result();
        result.setSuccess(true);

        if(dto == null || dto.getTaskInfo() ==null){
            result.setSuccess(false);
            result.setMessage("参数为空");
            return result;
        }

        OkrTaskInfo taskInfo = dto.getTaskInfo();
        if(taskInfo.getId() == null || StringUtils.isBlank(taskInfo.getTaskTitle())){
            result.setSuccess(false);
            result.setMessage("主键、任务标题均不可为空");
            return result;
        }

        try {
            okrService.updateTaskInfo(taskInfo);
        }catch(Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally {
            logger.info("okr/updateTaskInfo 方法结束");
        }

        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/getOkrInfo")
    public Result getOkrInfo(@RequestBody UserInfoDto dto,
                             HttpServletRequest request,
                             HttpServletResponse response){
        logger.info("okr/getOkrInfo 方法开始");
        Result<Map<String,Object>> result = new Result();
        result.setSuccess(true);

        try {
            Map<String, Object> map = okrService.getOkrInfo(dto.getUserId());
            result.setResult(map);
        }catch(Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally {
            logger.info("okr/getOkrInfo 方法结束");
        }

        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/saveOkrMember")
    public Result saveOkrMember(@RequestBody UserInfoDto dto,
                             HttpServletRequest request,
                             HttpServletResponse response){
        logger.info("/okr/saveOkrMember 方法开始");
        Result result = new Result();
        List<String> userIdList = dto.getUserIdList();
        if(userIdList == null || userIdList.size() == 0){
            result.setSuccess(false);
            result.setMessage("同步人员不能为空");
            return result;
        }

        try {
            okrService.insertOkrMember(userIdList);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/saveOkrMember 方法结束");
        }

        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/deleteOkrMember")
    public Result deleteOkrMember(@RequestBody UserInfoDto dto,
                                HttpServletRequest request,
                                HttpServletResponse response){
        logger.info("/okr/deleteOkrMember 方法开始");
        Result result = new Result();
        if(dto == null || StringUtils.isBlank(dto.getMemberId())){
            result.setSuccess(false);
            result.setMessage("需要删除的同步人不能为空");
            return result;
        }

        try {
            okrService.deleteOkrMember(dto.getMemberId());
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/deleteOkrMember 方法结束");
        }

        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/saveOkrMember2")
    public Result saveOkrMember2(@RequestBody UserInfoDto dto,
                                HttpServletRequest request,
                                HttpServletResponse response){
        logger.info("/okr/saveOkrMember2 方法开始");
        Result result = new Result();
        List<String> userIdList = dto.getUserIdList();
        if(userIdList == null || userIdList.size() == 0){
            result.setSuccess(false);
            result.setMessage("同步人员不能为空");
            return result;
        }

        List<String> userListFilter = new ArrayList<>();
        //同步人是否重复校验
        List<UserInfoDto> memberList = okrService.getMemberListByUserId(null);
        for(String userId:userIdList){
            int filter = 0;
            for (UserInfoDto memberInfo:memberList){
                if(StringUtils.equals(userId,memberInfo.getUserId())){
                    //result.setSuccess(false);
                    //result.setMessage(memberInfo.getName() + "已是同步人，请重新选择");
                    //return result;
                    filter = -1;
                    break;
                }
            }
            if(filter == 0){
                userListFilter.add(userId);
            }

        }

        try {
            okrService.insertOkrMember2(userListFilter);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/saveOkrMember2 方法结束");
        }

        return result;
    }

    /**
     * @param dto
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getMemberListByUserId")
    public Result<List<UserInfoDto>> getMemberListByUserId(@RequestBody UserInfoDto dto,
                                HttpServletRequest request,
                                HttpServletResponse response){
        logger.info("/okr/getMemberListByUserId 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        try {
            List<UserInfoDto> userInfoList = okrService.getMemberListByUserId(dto.getName());
            result.setResult(userInfoList);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/getMemberListByUserId 方法结束");
        }

        return result;
    }


    /**
     * 获取本季度同步给我OKR的同事的列表
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getSyncUserListByMemberId")
    public Result<Map<String, Object>> getSyncUserListByMemberId(@RequestBody UserInfoDto dto,
                                                           HttpServletRequest request,
                                                           HttpServletResponse response){
        logger.info("/okr/getSyncUserListByMemberId 方法开始");
        Result<Map<String, Object>> result = new Result();

        if(dto == null){
            result.setSuccess(false);
            result.setMessage("参数错误");
            return result;
        }

        try {
            Map<String, Object> resultMap = okrService.getSyncUserListByMemberId(dto.getYear(), dto.getQuarter(), dto.getName());
            result.setResult(resultMap);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/getSyncUserListByMemberId 方法结束");
        }

        return result;
    }


    /**
     * 保存状态指标
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/insertContentStatus")
    public Result<List<UserInfoDto>> insertContentStatus(@RequestBody OkrDto dto,
                                                         HttpServletRequest request,
                                                         HttpServletResponse response){
        logger.info("/okr/insertContentStatus 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        try {
            okrService.insertContentStatus(dto.getContent());
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/insertContentStatus 方法结束");
        }

        return result;
    }

    /**
     * 更新状态指标
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/updateContentStatus")
    public Result<List<UserInfoDto>> updateContentStatus(@RequestBody OkrDto dto,
                                                         HttpServletRequest request,
                                                         HttpServletResponse response){
        logger.info("/okr/updateContentStatus 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        try {
            okrService.updateContentStatus(dto.getContent());
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/updateContentStatus 方法结束");
        }

        return result;
    }

    /**
     * 同时保存O和krList
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/insertObjectAndKrList")
    public Result<List<UserInfoDto>> insertObjectAndKrList(@RequestBody OkrDto dto,
                                                         HttpServletRequest request,
                                                         HttpServletResponse response){
        logger.info("/okr/insertObjectAndKrList 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        try {
            okrService.insertObjectAndKrList(dto.getObjectInfo(), dto.getKrList());
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/insertObjectAndKrList 方法结束");
        }

        return result;
    }

    /**
     * 更新状态指标
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/updateObject")
    public Result<List<UserInfoDto>> updateObject(@RequestBody OkrDto dto,
                                                           HttpServletRequest request,
                                                           HttpServletResponse response){
        logger.info("/okr/updateObject 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        if(dto == null || dto.getObjectInfo()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrObjectInfo objectInfo = dto.getObjectInfo();
        if(objectInfo.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.updateObject(objectInfo);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/updateObject 方法结束");
        }

        return result;
    }


    /**
     * 更新状态指标
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/updateKr")
    public Result<List<UserInfoDto>> updateKr(@RequestBody OkrDto dto,
                                                  HttpServletRequest request,
                                                  HttpServletResponse response){
        logger.info("/okr/updateKr 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        if(dto == null || dto.getKrInfo()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrKrInfo krInfo = dto.getKrInfo();
        if(krInfo.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.updateKr(krInfo);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/updateKr 方法结束");
        }

        return result;
    }


    /**逻辑删除Object
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/deleteObjectInfoById")
    public Result<List<UserInfoDto>> deleteObjectInfoById(@RequestBody OkrDto dto,
                                                  HttpServletRequest request,
                                                  HttpServletResponse response){
        logger.info("/okr/deleteObjectInfoById 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        if(dto == null || dto.getObjectInfo()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrObjectInfo objectInfo = dto.getObjectInfo();
        if(objectInfo.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.deleteObjectInfoById(objectInfo);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/deleteObjectInfoById 方法结束");
        }

        return result;
    }

    /**逻辑删除Kr
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/deleteKrById")
    public Result<List<UserInfoDto>> deleteKrById(@RequestBody OkrDto dto,
                                                          HttpServletRequest request,
                                                          HttpServletResponse response){
        logger.info("/okr/deleteKrById 方法开始");
        Result<List<UserInfoDto>> result = new Result();

        if(dto == null || dto.getKrInfo()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrKrInfo krInfo = dto.getKrInfo();
        if(krInfo.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.deleteKrById(krInfo);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/deleteKrById 方法结束");
        }

        return result;
    }

    @ResponseBody
    @RequestMapping(value = "/deleteContentStatusById")
    public Result deleteContentStatuaById(@RequestBody OkrDto dto,
                                          HttpServletRequest request,
                                          HttpServletResponse response){
        logger.info("/okr/deleteContentStatusById 方法开始");
        Result result = new Result();

        if(dto == null || dto.getContent()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrContentStatus content = dto.getContent();
        if(content.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.deleteContentStatusById(content);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/deleteContentStatusById 方法结束");
        }

        return result;

    }

    @ResponseBody
    @RequestMapping(value = "/deleteTaskInfoById")
    public Result deleteTaskInfoById(@RequestBody OkrDto dto,
                                          HttpServletRequest request,
                                          HttpServletResponse response){
        logger.info("/okr/deleteTaskInfoById 方法开始");
        Result result = new Result();

        if(dto == null || dto.getTaskInfo()==null ){
            result.setSuccess(false);
            result.setMessage("参数不能为空");
            return result;
        }
        OkrTaskInfo taskInfo = dto.getTaskInfo();
        if(taskInfo.getId() == null){
            result.setSuccess(false);
            result.setMessage("主键不能为空");
            return result;
        }

        try {
            okrService.deleteTaskInfoById(taskInfo);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/deleteTaskInfoById 方法结束");
        }

        return result;
    }

    /**
     * 保存状态指标
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/insertOkrKrInfo")
    public Result insertOkrKrInfo(@RequestBody OkrDto dto,
                                                         HttpServletRequest request,
                                                         HttpServletResponse response){
        logger.info("/okr/insertOkrKrInfo 方法开始");
        Result result = new Result();

        if(dto == null || dto.getKrInfo() ==null){
            result.setSuccess(false);
            result.setMessage("参数不可为空");
            return result;
        }

        OkrKrInfo krInfo = dto.getKrInfo();

        if(krInfo.getObjectId() == null || krInfo.getCompletion() == null || StringUtils.isBlank(krInfo.getKrInfo())){
            result.setSuccess(false);
            result.setMessage("参数不可为空");
            return result;
        }

        try {
            okrService.insertOkrKrInfo(dto.getKrInfo());
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/insertOkrKrInfo 方法结束");
        }

        return result;
    }


    /**
     * 获取OKR所有年
     * @param request
     * @param response
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/getObjectYear")
    public Result getObjectYear(@RequestBody OkrDto dto,
                                  HttpServletRequest request,
                                  HttpServletResponse response){
        logger.info("/okr/getObjectYear 方法开始");
        Result<List<Integer>> result = new Result();

        try {
            List<Integer> yearList = okrService.getObjectYear();
            result.setResult(yearList);
        }catch (Exception e){
            logger.error(e.getMessage());
            result.setSuccess(false);
            result.setMessage("error");
        }finally{
            logger.info("/okr/getObjectYear 方法结束");
        }

        return result;
    }

}
