package cn.com.ctop.okr.service.impl;

import cn.com.ctop.okr.dto.UserInfoDto;
import cn.com.ctop.okr.entity.*;
import cn.com.ctop.okr.mapper.*;
import cn.com.ctop.okr.service.OkrService;
import cn.com.ctop.okr.utils.DateUtils;
import cn.com.ctop.okr.vo.LoginUser;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Year;
import java.util.*;

@Service
@Slf4j
public class OkrServiceImpl implements OkrService {
    @Autowired
    private OkrContentStatusMapper okrContentStatusMapper;
    @Autowired
    private OkrKrInfoMapper okrKrInfoMapper;
    @Autowired
    private OkrLogMapper okrLogMapper;
    @Autowired
    private OkrMemberMapper okrMemberMapper;
    @Autowired
    private OkrObjectInfoMapper okrObjectInfoMapper;
    @Autowired
    private OkrTaskInfoMapper okrTaskInfoMapper;
    @Autowired
    private QYWxUserInfoMapper wxUserInfoMapper;

    @Override
    public Map<String, Object> getOkrInfo(String userId){
        Map<String, Object> map = new HashMap<>();
        if(StringUtils.isBlank(userId)){
            QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
            userId = sysUser.getOpenId();
            //userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        }

        //季度
        Map<String, Object> timeMap = new HashMap<>();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        timeMap.put("year",year);
        timeMap.put("quarter",quarter);

        //本周任务信息
        List<OkrTaskInfo> taskInfoList = okrTaskInfoMapper.getTaskInfo(userId, year, quarter, 1);

        //未来四周任务信息
        List<OkrTaskInfo> taskFourWeekList = okrTaskInfoMapper.getTaskInfo(userId, year, quarter, 2);

        //OKR信息
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoMapper.getQuarterObject(userId, year, quarter);
        List<Map<String, Object>> okrList = new ArrayList<>();
        for (OkrObjectInfo okrObjectInfo:OkrObjectInfoList){
            Map<String, Object> okrMap = new HashMap<>();
            List<OkrKrInfo> OkrKrInfoList = okrKrInfoMapper.getKrByObjectId(okrObjectInfo.getId(),userId, year, quarter);
            okrMap.put("objectInfo",okrObjectInfo);
            okrMap.put("krList",OkrKrInfoList);
            okrList.add(okrMap);
        }

        //状态信息
        List<OkrContentStatus> contentStatusList = okrContentStatusMapper.getContentStatusList(userId, year, quarter);

        map.put("time", timeMap);
        map.put("taskInfoList",taskInfoList);
        map.put("taskFourWeekList",taskFourWeekList);
        map.put("okr",okrList);
        map.put("status", contentStatusList);

        return map;
    }


    @Override
    public void insertOkrMember(List<String> memberList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        okrMemberMapper.deleteOkrMemberByUserId(userId,year,quarter);
        okrMemberMapper.insertOkrMember(userId, memberList, year, quarter);
    }

    @Override
    public void insertOkrMember2(List<String> memberList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        //okrMemberMapper.deleteOkrMemberByUserId2(userId,memyear,quarter);
        okrMemberMapper.insertOkrMember(userId, memberList, year, quarter);
    }

    @Override
    public void deleteOkrMember(String memberId){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        okrMemberMapper.deleteOkrMemberByUserId2(userId, memberId, year, quarter);

    }

    @Override
    public List<UserInfoDto> getMemberListByUserId(String name){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        return okrMemberMapper.getMemberListByUserId(userId, name, year, quarter);
    }

    //保存任务
    @Override
    public void insertTaskInfo(OkrTaskInfo taskInfo){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        System.out.println("sysUser:"+ sysUser.getOpenId() + " name:" + sysUser.getName() );

        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        String userId = sysUser.getOpenId();
        Integer year = DateUtils.getYear(new Date());
        Integer quarter = DateUtils.getQuarter(new Date());
        taskInfo.setUserId(userId);
        taskInfo.setYear(year);
        taskInfo.setQuarter(quarter);
        okrTaskInfoMapper.insertTaskInfo(taskInfo);

        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(1); //操作：1.创建2.修改3.删除
        if(taskInfo.getType() == 1){
            okrLog.setOperation(3);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        }else{
            okrLog.setOperation(4);
        }
        okrLog.setOperationId(taskInfo.getId());
        okrLog.setYear(year);
        okrLog.setQuarter(quarter);
        okrLog.setContent(JSONObject.toJSONString(taskInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    //更新任务
    @Override
    public void updateTaskInfo(OkrTaskInfo taskInfo){
        okrTaskInfoMapper.updateTaskInfo(taskInfo);
        taskInfo = okrTaskInfoMapper.selectById(taskInfo.getId());

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(2); //操作：1.创建2.修改3.删除
        if(taskInfo.getType() == 1){
            okrLog.setOperation(3);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        }else{
            okrLog.setOperation(4);
        }
        okrLog.setOperationId(taskInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(taskInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    //保存任务
    @Override
    public void insertContentStatus(OkrContentStatus content){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        content.setUserId(userId);
        content.setYear(DateUtils.getYear(new Date()));
        content.setQuarter(DateUtils.getQuarter(new Date()));
        okrContentStatusMapper.insertContentStatus(content);

        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(1); //操作：1.创建2.修改3.删除
        okrLog.setOperation(5);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(content.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(content));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    //更新任务
    @Override
    public void updateContentStatus(OkrContentStatus content){
        okrContentStatusMapper.updateContentStatus(content);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(2); //操作：1.创建2.修改3.删除
        okrLog.setOperation(5);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(content.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(content));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    //同时保存O和krList
    @Override
    public void insertObjectAndKrList(OkrObjectInfo objectInfo, List<OkrKrInfo> krList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        objectInfo.setYear(year);
        objectInfo.setQuarter(quarter);
        objectInfo.setUserId(userId);
        okrObjectInfoMapper.insertObjectInfo(objectInfo);
        //日志留存
        OkrLog okrObjectLog = new OkrLog();
        okrObjectLog.setType(1); //操作：1.创建2.修改3.删除
        okrObjectLog.setOperation(1);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrObjectLog.setOperationId(objectInfo.getId());
        okrObjectLog.setYear(DateUtils.getYear(new Date()));
        okrObjectLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrObjectLog.setContent(JSONObject.toJSONString(objectInfo));
        okrObjectLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrObjectLog);
        //

        // 由于需要返回主键id作为日志留存，故放弃批量插入的接口
        // okrKrInfoMapper.insertBatchOkrKrInfo(krList,userId, objectInfo.getId(),year,quarter);
        for(OkrKrInfo kr:krList){
            kr.setUserId(userId);
            kr.setObjectId(objectInfo.getId());
            kr.setYear(year);
            kr.setQuarter(quarter);
            okrKrInfoMapper.insertOkrKrInfo(kr);

            //日志留存
            OkrLog okrKrLog = new OkrLog();
            okrKrLog.setType(1); //操作：1.创建2.修改3.删除
            okrKrLog.setOperation(2);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
            okrKrLog.setOperationId(kr.getId());
            okrKrLog.setYear(year);
            okrKrLog.setQuarter(quarter);
            okrKrLog.setContent(JSONObject.toJSONString(kr));
            okrKrLog.setUserId(userId);
            okrLogMapper.insertOkrLog(okrKrLog);
        }
    }

    @Override
    public void updateObject(OkrObjectInfo objectInfo){
        okrObjectInfoMapper.updateObjectInfo(objectInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(2); //操作：1.创建2.修改3.删除
        okrLog.setOperation(1);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(objectInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(objectInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    @Override
    public void updateKr(OkrKrInfo okrKrInfo){
        okrKrInfoMapper.updateKrInfo(okrKrInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(2); //操作：1.创建2.修改3.删除
        okrLog.setOperation(2);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(okrKrInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(okrKrInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    //获取本季度同步给我OKR的同事的列表
    @Override
    public Map<String, Object> getSyncUserListByMemberId(Integer year, Integer quarter, String name){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String memberId = sysUser.getOpenId();
        //String memberId = "obQGf1fqw5_RJGrLzhWBioxUH0T0";
        Integer isCurrentQuarter = null;
        Map<String, Object> map = new HashMap<>();
        //List<List<Map<String, Object>>> list = new ArrayList<>();

        Integer todayQuarter = DateUtils.getQuarter(new Date());
        Integer todayYear = DateUtils.getYear(new Date());
        if(year ==null || quarter ==null){
            year = todayYear;
            quarter = todayQuarter;
        }

        //如果不是当年当季度，返回O和KR的信息；是当季度，直接返回OKR四象限信息
        if(todayYear.equals(year) && todayQuarter.equals(quarter)){
            isCurrentQuarter = 1;
        }else{
            isCurrentQuarter = 0;
        }

        List<String> userList = okrMemberMapper.getSyncUserListByMemberId(memberId, year, quarter, name);
        List<Map<String, Object>> mapList = new ArrayList<>();
        if(userList!=null && userList.size()!=0){
            List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoMapper.getAllQuarterObject(userList, year, quarter);

            for(OkrObjectInfo object:OkrObjectInfoList){
                Map<String, Object> okrmap = new HashMap<>();
                List<OkrKrInfo> krList = okrKrInfoMapper.getKrByObjectId(object.getId(), object.getUserId(), year, quarter);
                QYWxUserInfo userInfo = wxUserInfoMapper.getUserInfoByOpenId(object.getUserId());
                okrmap.put("krList", krList);
                okrmap.put("object", object);
                okrmap.put("name",userInfo.getName());
                mapList.add(okrmap);
            }

        }

        map.put("isCurrentQuarter",isCurrentQuarter);
        map.put("list",mapList);
        map.put("year",todayYear);
        map.put("quarter",todayQuarter);

        return map;
    }


    @Override
    public void deleteObjectInfoById(OkrObjectInfo objectInfo){
        okrObjectInfoMapper.deleteObjectInfoById(objectInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(3); //操作：1.创建2.修改3.删除
        okrLog.setOperation(1);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(objectInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(objectInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    @Override
    public void deleteKrById(OkrKrInfo krInfo){
        okrKrInfoMapper.deleteKrById(krInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(3); //操作：1.创建2.修改3.删除
        okrLog.setOperation(2);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(krInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(krInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    @Override
    public void deleteContentStatusById(OkrContentStatus content){
        okrContentStatusMapper.deleteContentStatuaById(content);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(3); //操作：1.创建2.修改3.删除
        okrLog.setOperation(5);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrLog.setOperationId(content.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(content));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    @Override
    public void deleteTaskInfoById(OkrTaskInfo okrTaskInfo){
        okrTaskInfoMapper.deleteTaskInfoById(okrTaskInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";
        //日志留存
        OkrLog okrLog = new OkrLog();
        okrLog.setType(3); //操作：1.创建2.修改3.删除
        if(okrTaskInfoMapper.selectById(okrTaskInfo.getId()).getType()==1){
            okrLog.setOperation(3);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        }else{
            okrLog.setOperation(4);
        }

        okrLog.setOperationId(okrTaskInfo.getId());
        okrLog.setYear(DateUtils.getYear(new Date()));
        okrLog.setQuarter(DateUtils.getQuarter(new Date()));
        okrLog.setContent(JSONObject.toJSONString(okrTaskInfo));
        okrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrLog);
    }

    @Override
    public void insertOkrKrInfo(OkrKrInfo krInfo){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //String userId = "ou_03c4826ae66623a53f2af456ff3661a1";

        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());

        krInfo.setYear(year);
        krInfo.setQuarter(quarter);
        krInfo.setUserId(userId);
        okrKrInfoMapper.insertOkrKrInfo(krInfo);

        //日志留存
        OkrLog okrKrLog = new OkrLog();
        okrKrLog.setType(1); //操作：1.创建2.修改3.删除
        okrKrLog.setOperation(2);  //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        okrKrLog.setOperationId(krInfo.getId());
        okrKrLog.setYear(year);
        okrKrLog.setQuarter(quarter);
        okrKrLog.setContent(JSONObject.toJSONString(krInfo));
        okrKrLog.setUserId(userId);
        okrLogMapper.insertOkrLog(okrKrLog);

    }

    @Override
    public List<Integer> getObjectYear(){
        return okrObjectInfoMapper.getObjectYear();
    }



}
